unit Common.Logging.Impl.Classes;

interface

uses
  System.Classes, System.SysUtils,
  Common.Logging;

type
  TDebugLogger = class(TInterfacedObject,ILogger)
  public
    procedure Log(level: TLogLevel; const text: String);
    procedure Error(const text: String; E: Exception = nil);
    procedure Info(const text: String);
    procedure Warn(const text: String);
    procedure Debug(const text: String);
  end;

  TStringsLogger = class(TInterfacedObject,ILogger)
  private
    FOutputRef: TStrings;
  public
    constructor Create(const output: TStrings);
    procedure Log(level: TLogLevel; const text: String);
    procedure Error(const text: String; E: Exception = nil);
    procedure Info(const text: String);
    procedure Warn(const text: String);
    procedure Debug(const text: String);
  end;


  TFileLogger = class(TInterfacedObject,ILogger)
  private
    FFilename: String;
    FStreamWriter: TStreamWriter;
    procedure WriteLine(const text: string);
  public
    constructor Create(const LogFilename: String);
    destructor Destroy; override;
    procedure Log(level: TLogLevel; const text: String);
    procedure Error(const text: String; E: Exception = nil);
    procedure Info(const text: String);
    procedure Warn(const text: String);
    procedure Debug(const text: String);
  end;



implementation

uses
  Winapi.Windows;

const
  sLogLevels: Array[TLogLevel] of string = ('Debug', 'Info', 'Warn', 'Error');

{ TDebugLogger }

procedure TDebugLogger.Debug(const text: String);
begin
  Log(llDebug, text);
end;

procedure TDebugLogger.Error(const text: String; E: Exception);
begin
  if Assigned(E) then
    Log(llError, text + Format('Exception %s with Message %s',[E.ClassName,E.Message]))
  else
    Log(llError, text);
end;

procedure TDebugLogger.Info(const text: String);
begin
  Log(llInfo, text);
end;

procedure TDebugLogger.Log(level: TLogLevel; const text: String);
begin
  OutPutDebugString(PChar(sLogLevels[level] + ': ' + text));
end;

procedure TDebugLogger.Warn(const text: String);
begin
  Log(llWarn, text);
end;

{ TStringsLogger }

constructor TStringsLogger.Create(const output: TStrings);
begin
  inherited create;
  if output=nil then
    raise EArgumentNilException.Create('Expected Value for output');
  FOutputRef := output;
end;

procedure TStringsLogger.Debug(const text: String);
begin
  Log(llDebug, text);
end;

procedure TStringsLogger.Error(const text: String; E: Exception);
begin
   if Assigned(E) then
    Log(llError, text + Format('Exception %s with Message %s',[E.ClassName,E.Message]))
  else
    Log(llError, text);
end;

procedure TStringsLogger.Info(const text: String);
begin
  Log(llInfo, text);
end;

procedure TStringsLogger.Log(level: TLogLevel; const text: String);
begin
  FOutputRef.Add(sLogLevels[level] + ': ' + text);
end;

procedure TStringsLogger.Warn(const text: String);
begin
  Log(llWarn, text);
end;

{ TFileLogger }

constructor TFileLogger.Create(const LogFilename: String);
begin
  inherited Create;
  FFilename := LogFilename;
  FStreamWriter := TStreamWriter.create(FFilename,true);
  FStreamWriter.AutoFlush := True;
  FStreamWriter.NewLine := sLineBreak;
end;

procedure TFileLogger.Debug(const text: String);
begin
  Log(llDebug,text);
end;

destructor TFileLogger.Destroy;
begin
  FStreamWriter.Free;
  inherited;
end;

procedure TFileLogger.Error(const text: String; E: Exception);
begin
  if Assigned(E) then
    Log(llError, text + Format('Exception %s with Message %s',[E.ClassName,E.Message]))
  else
    Log(llError, text);
end;

procedure TFileLogger.Info(const text: String);
begin
  Log(llInfo,text);
end;

procedure TFileLogger.Log(level: TLogLevel; const text: String);
begin
  WriteLine(sLogLevels[level] + ': ' + text);
end;

procedure TFileLogger.Warn(const text: String);
begin
   Log(llWarn,text);
end;

procedure TFileLogger.WriteLine(const text: string);
begin
  FStreamWriter.WriteLine(DateTimeToStr(now) + ' ' + text);
end;

end.
