unit Shopping.ShoppingBasket;

interface

uses
  System.Classes, System.SysUtils, System.Types, System.Generics.Collections,
  Spring.Collections,
  Common.Logging, Common.Logging.Classes,
  Shopping.Types;

type
  TShoppingBasket = class(TObject)
  strict private
    FItemsCalculator: IVoucherCalculator;
    FLogger: ILogger;
    FVoucherCodes: IList<string>;
    FItems: IList<TBasketItem>;
    FCustomerId: string;
    FTotalValue: Currency;
    FTotalDiscount: Currency;
    FHints: TStrings;
    procedure ResetResults;
    function GetItemsPrice: Currency;
    function CalculateDiscount(const itemsPrice: Currency): Currency;
  strict protected
    procedure DoChange; virtual;
  public
    constructor Create(const logger: ILogger; const customerId: string; const voucherCalculator: IVoucherCalculator);
    destructor Destroy; override;
    procedure AddVoucher(const code: string);
    procedure AddVouchers(const codes: TStringDynArray);
    procedure AddItem(const item: TBasketItem);
    procedure AddItems(const items: TBasketItemsArray);
    procedure Load(const loader: IBasketLoader);
    procedure Calculate;
    property TotalValue: Currency read FTotalValue;
    property TotalDiscount : Currency read FTotalDiscount;
    property Hints: TStrings read FHints;
  end;


implementation

{ TShoppingBasket }

procedure TShoppingBasket.AddItem(const Item: TBasketItem);
begin
  FItems.Add(Item);
  DoChange;
end;

procedure TShoppingBasket.AddItems(const items: TBasketItemsArray);
begin
  FItems.AddRange(items);
  DoChange;
end;

procedure TShoppingBasket.AddVoucher(const Code: string);
begin
  FVoucherCodes.Add(code);
  DoChange;
end;


procedure TShoppingBasket.AddVouchers(const codes: TStringDynArray);
begin
  FVoucherCodes.AddRange(codes);
  DoChange;
end;

procedure TShoppingBasket.Calculate;
begin
  ResetResults;
  FTotalValue := GetItemsPrice;
  FTotalDiscount := CalculateDiscount(FTotalValue);
  // Limit the Discount
  if FTotalDiscount > FTotalValue then
  begin
    FTotalValue := 0;
    FTotalDiscount := FTotalValue;
    FLogger.Warn('Discount bigger than Value');
  end
  else
    FTotalValue := FTotalValue - FTotalDiscount;
end;


function TShoppingBasket.CalculateDiscount(const itemsPrice: Currency): Currency;
const
  fmt_invalid = 'Voucher %s not acccepted : %s';
var
  voucher: string;
  vouchervalue: Currency;
begin
  FLogger.Debug('Calculating discount');
  Result := 0;
  for voucher in FVoucherCodes do
  begin
    try
      vouchervalue := FItemsCalculator.CalculateVoucher(itemsPrice, FCustomerId, voucher);
      Result := FTotalDiscount + vouchervalue;
    except
      On E: EInvalidVoucher do
      begin
        var errmsg: string := Format(fmt_invalid, [voucher, E.Message]);
        FLogger.Warn(errmsg);
        FHints.Add(errmsg)
      end;
      On E: Exception do
      begin
        FLogger.Error(E.Message);
        FHints.Text := 'An internal error occurred calculating your vouchers';
        raise;
      end;
    end;
  end;
end;


function TShoppingBasket.GetItemsPrice: Currency;
begin
  Result := 0;
  FLogger.Debug('Calculating Items Price');
  for var index: integer := 0 to FItems.Count - 1 do
  begin
    Result := Result + FItems[index].Count*FItems[index].Price;
  end;
end;


constructor TShoppingBasket.Create(const logger: ILogger; const customerId: string;
  const voucherCalculator: IVoucherCalculator);
begin
  inherited Create;
  FLogger := logger;
  FItemsCalculator := voucherCalculator;
  FCustomerId := customerId;
  FItems :=  TCollections.CreateList<TBasketItem>;
  FVoucherCodes := TCollections.CreateList<string>;
  FHints := TStringList.Create;
end;

destructor TShoppingBasket.Destroy;
begin
  FHints.Free;
  inherited;
end;

procedure TShoppingBasket.DoChange;
begin
  //Calculate;
end;

procedure TShoppingBasket.Load(const loader: IBasketLoader);
var
  content: TBasketContent;
begin
  content := loader.LoadItems(FCustomerId);
  FItems.Clear;
  FItems.InsertRange(0, content.BasketItems);
  FVoucherCodes.Clear;
  FVoucherCodes.InsertRange(0, content.VoucherCodes);
end;

procedure TShoppingBasket.ResetResults;
begin
  FLogger.Debug('ResetResults');
  FHints.Clear;
  FTotalValue := 0;
  FTotalDiscount := 0;
end;

end.
