unit Tests.Shopping.ShoppingBasket;

interface

uses
  TestFramework,
  Common.Logging,
  Shopping.ShoppingBasket, Shopping.Types,
  System.Generics.Collections, System.SysUtils,
  Spring.Collections,
  Spring.Mocking,
  System.Types, System.Classes;

type
  ///<summary>Logger for null-Object-Pattern<summary>
  TNullLogger = class(TInterfacedObject,ILogger)
  public
    procedure Log(level: TLogLevel; const msg: String);
    procedure Error(const text: String; E: Exception = nil);
    procedure Info(const text: String);
    procedure Warn(const text: String);
    procedure Debug(const text: String);
  end;

const
  SingleItem : TBasketItem = (Count: 1;  Price: 200;);
  OtherItem : TBasketItem = (Count: 1;  Price: 100;);
  Count10Item : TBasketItem = (Count: 10;  Price: 100;);

type
  // Testmethods for class TShoppingBasket
  TestTShoppingBasket_Calculate = class(TTestCase)
  strict private
    FShoppingBasket: TShoppingBasket;
    FVoucherMock: Mock<IVoucherCalculator>;
  public
    procedure SetUp; override;
    procedure TearDown; override;
  published
    procedure Test_EmptyBasket;
    procedure Test_SingleItem;
    procedure Test_ItemCount;
    procedure Test_ManyItems;
    procedure Test_OneVoucher;
    procedure Test_OneVoucher_Behavior;
    procedure Test_OneVoucher_Received;
  end;

implementation


{ TNullLogger }

procedure TNullLogger.Debug(const text: String);
begin
  // nothing to do
end;

procedure TNullLogger.Error(const text: String; E: Exception);
begin
  // nothing to do
end;

procedure TNullLogger.Info(const text: String);
begin
  // nothing to do
end;

procedure TNullLogger.Log(level: TLogLevel; const msg: String);
begin
  // nothing to do
end;

procedure TNullLogger.Warn(const text: String);
begin
  // nothing to do
end;



procedure TestTShoppingBasket_Calculate.SetUp;
begin
  FVoucherMock := Mock<IVoucherCalculator>.create(TMockBehavior.Strict);
  FShoppingBasket := TShoppingBasket.Create(TNullLogger.Create, '', FVoucherMock);
end;

procedure TestTShoppingBasket_Calculate.TearDown;
begin
  FShoppingBasket.Free;
  FVoucherMock.Free;
end;


procedure TestTShoppingBasket_Calculate.Test_EmptyBasket;
begin
  // Arrange : nothing to do
  // Act
  FShoppingBasket.Calculate;
  // Assert
  CheckEquals(0,FShoppingBasket.TotalValue);
end;


procedure TestTShoppingBasket_Calculate.Test_SingleItem;
begin
  // Arrange
  FShoppingBasket.AddItem(SingleItem);
  // Act
  FShoppingBasket.Calculate;
  // Assert
  CheckEquals(SingleItem.Price,FShoppingBasket.TotalValue);
end;


procedure TestTShoppingBasket_Calculate.Test_ItemCount;
begin
  // Arrange
  FShoppingBasket.AddItem(Count10Item);
  // Act
  FShoppingBasket.Calculate;
  // Assert
  CheckEquals(Count10Item.Price*Count10Item.Count,FShoppingBasket.TotalValue);
end;


procedure TestTShoppingBasket_Calculate.Test_ManyItems;
begin
  // Arrange
  FShoppingBasket.AddItem(Count10Item);
  FShoppingBasket.AddItem(OtherItem);
  FShoppingBasket.AddItem(SingleItem);
  // Act
  FShoppingBasket.Calculate;
  // Assert
  CheckEquals(Count10Item.Price*Count10Item.Count + OtherItem.Price + SingleItem.Price ,FShoppingBasket.TotalValue);
end;


procedure TestTShoppingBasket_Calculate.Test_OneVoucher;
const
  voucherCode = 'XDRET13';
  voucherValue = 20;
begin
  // Arrange
  FShoppingBasket.AddItem(SingleItem);
  FShoppingBasket.AddVoucher(voucherCode);
  FVoucherMock.Setup.Returns<Currency>(voucherValue).When.CalculateVoucher(SingleItem.Price,'',voucherCode);
  // Act
  FShoppingBasket.Calculate;
  // Assert
  CheckEquals(voucherValue, FShoppingBasket.TotalDiscount);
end;


procedure TestTShoppingBasket_Calculate.Test_OneVoucher_Behavior;
const
  voucherCode = 'XDRET13';
  voucherValue = 20;
begin
  // Arrange
  FShoppingBasket.AddItem(SingleItem);
  FShoppingBasket.AddVoucher(voucherCode);
  FVoucherMock.Behavior := TMockBehavior.Strict;
  FVoucherMock.Setup.Returns<Currency>(voucherValue).When.CalculateVoucher(SingleItem.Price,'',voucherCode);
  // Act
  FShoppingBasket.Calculate;
  // Assert
  CheckEquals(voucherValue, FShoppingBasket.TotalDiscount);
end;

procedure TestTShoppingBasket_Calculate.Test_OneVoucher_Received;
const
  voucherCode = 'XDRET13';
  voucherValue = 20;
begin
  // Arrange
  FShoppingBasket.AddItem(SingleItem);
  FShoppingBasket.AddVoucher(voucherCode);
  FVoucherMock.Setup.Returns<Currency>(voucherValue).When.CalculateVoucher(SingleItem.Price,'',voucherCode);
  // Act
  FShoppingBasket.Calculate;
  // Assert
  FVoucherMock.Received(Times.Once).CalculateVoucher(SingleItem.Price,'',voucherCode);
  CheckEquals(voucherValue, FShoppingBasket.TotalDiscount);
end;

initialization
  // Alle Testflle beim Testprogramm registrieren
  RegisterTest(TestTShoppingBasket_Calculate.Suite);
end.

